local color_picker = {}

local color_original = {0, 0, 0}
local active_color = {}

local xres = gkinterface.GetXResolution()
local yres = gkinterface.GetYResolution()
local swatch_size = math.floor(math.min(xres/67, yres/42))

--group related sliders by domain: when a slider is updated, change sliders from other domains to match
local color_domains = {
	RGB = {"red", "green", "blue"},
	HSV = {"hue", "saturation", "value"},
}

--slider min/max values
local slider_ranges = {
	red = {0,255},
	green = {0,255},
	blue = {0,255},
	hue = {0,360},
	saturation = {0,100},
	value = {0,100},
}

hsv_palette = {
	"255 127 127",
	"255 170 127",
	"255 212 127",
	"255 255 127",
	"212 255 127",
	"170 255 127",
	"127 255 127",
	"127 255 170",
	"127 255 212",
	"127 255 255",
	"127 212 255",
	"127 170 255",
	"127 127 255",
	"169 127 255",
	"212 127 255",
	"255 127 255",
	"255 127 212",
	"255 127 169",
	"255 255 255",

	"255 0 0",
	"255 84 0",
	"255 170 0",
	"255 255 0",
	"170 255 0",
	"84 255 0",
	"0 255 0",
	"0 255 85",
	"0 255 169",
	"0 255 255",
	"0 169 255",
	"0 85 255",
	"0 0 255",
	"84 0 255",
	"170 0 255",
	"255 0 255",
	"255 0 170",
	"255 0 84",
	"127 127 127",

	"127 0 0",
	"127 42 0",
	"127 85 0",
	"127 127 0",
	"85 127 0",
	"42 127 0",
	"0 127 0",
	"0 127 42",
	"0 127 84",
	"0 127 127",
	"0 84 127",
	"0 42 127",
	"0 0 127",
	"42 0 127",
	"85 0 127",
	"127 0 127",
	"127 0 85",
	"127 0 42",
	"0 0 0",
}

function color_picker:create()
	local dlg
	local cc = { --color controls
		label = {},
		slider = {},
		edit = {}
	}
	local color_preview
	local color_palette = {}
	local okbutton, resetbutton, cancelbutton
	
	 ---------------------
	--  Local Functions  --
	 ---------------------
	
	local function normalize_rgb(r,g,b)
		if type(r)=="table" then --table values don't have to be integers
			r,g,b = r[1],r[2],r[3] --g & b inputs ignored
		end --elseif type(r)=="string" then r,g,b = r:match("^(%d+) (%d+) (%d+)") end --TODO
	
		r = tonumber(r)
		g = tonumber(g)
		b = tonumber(b)
		
		assert(r and g and b) --TODO error msg
	
		r = math.min(math.max(r,0), 255)
		g = math.min(math.max(g,0), 255)
		b = math.min(math.max(b,0), 255)
	
		r = r/255
		g = g/255
		b = b/255
	
		return r,g,b
	end
	
	--h (hue) 0-360, s (saturation) 0-1, v (value) 0-1
	local function rgb2hsv(r,g,b)
		r,g,b = normalize_rgb(r,g,b)
	
		local min_val = math.min(r,g,b)
		local max_val = math.max(r,g,b)
		local delta = max_val - min_val
	
		local v = max_val
	
		if min_val==max_val then return {0, 0, v} end
	
		local s = delta/max_val
	
		local h
		if max_val==r then
			h = ((g-b)/delta) % 6
		elseif max_val==g then
			h = (b-r)/delta + 2
		else
			h = (r-g)/delta + 4
		end
	
		return {60*h, s, v}
	end

	local hue_map = {
		{1,4,2}, --   0 <= h < 60
		{3,1,2}, --  60 <= h < 120
		{2,1,4}, -- 120 <= h < 180
		{2,3,1}, -- 180 <= h < 240
		{4,2,1}, -- 240 <= h < 300
		{1,2,3}, -- 300 <= h < 360
	}
	local function hsv2rgb(h,s,v)
		--TODO validation function
		if s==0 then
			local grey = v*255
			return {grey, grey, grey}
		end
	
		if h==360 then h=0 end
		local hue_index = math.floor(h/60)
		local delta = h/60 - hue_index
	
		local vals = {
			v,
			v*(1-s),
			v*(1-s*delta),
			v*(1-s*(1-delta)),
		}
	
		local map = hue_map[hue_index+1]
		return {255*vals[ map[1] ], 255*vals[ map[2] ], 255*vals[ map[3] ]}
	end
	
	local function color2rgb(color, domain)
		assert(type(color)=="table") --TODO error msg
		if not color_domains[domain] then return false end --invalid domain
		
		if domain=="RGB" then
			return color --no conversion necessary
		elseif domain=="HSV" then
			local h = tonumber(color[1])
			local s = tonumber(color[2])
			local v = tonumber(color[3])
			
			if not h or not s or not v then return false end
			
			h = math.min(math.max(h, 0), 360)
			s = math.min(math.max(s/100, 0), 1)
			v = math.min(math.max(v/100, 0), 1)
			
			local rgb = hsv2rgb(h,s,v)
			for i=1,3 do
				rgb[i] = math.round(rgb[i])
			end
			
			return rgb
		end
	end
	
	--update dialog to reflect new color
	local function color_update()
		--update color preview
		local r,g,b = unpack(active_color)
		local color_str = string.format("%d %d %d", r,g,b)
		color_preview.bgcolor = color_str.." 255 &"
		
		--update RGB controls
		cc.slider.red.posx = r
		cc.slider.green.posx = g
		cc.slider.blue.posx = b
		
		cc.edit.red.value = r
		cc.edit.green.value = g
		cc.edit.blue.value = b
		
		--convert to hsv
		local h,s,v = unpack(rgb2hsv(r,g,b))
		
		h = math.round(h)
		s = math.round(100*s)
		v = math.round(100*v)
		
		--update HSV controls
		cc.slider.hue.posx = h
		cc.slider.saturation.posx = s
		cc.slider.value.posx = v
		
		cc.edit.hue.value = h
		cc.edit.saturation.value = s
		cc.edit.value.value = v
		
		resetbutton.active = "YES"
	end
	
	--returns table with 3 entries for values of sliders of given domain
	local function get_slider_values(domain)
		local slider_names = color_domains[domain]
		if not slider_names then return end
		
		local values = {}
		for _,name in ipairs(slider_names) do
			local slider = cc.slider[name]
			if not slider then return end
			
			table.insert(values, slider.posx or 0)
		end
		
		return values
	end
	
	local function slider_action(self)
		local domain = self.domain
		
		local new_color = get_slider_values(domain)
		if not new_color then return end
		
		local rgb = color2rgb(new_color, domain)
		if not rgb then return end
		
		active_color = rgb
		color_update()
	end
	
	local function edit_update(self)
		local new_val = tonumber(self.value)
		
		if new_val then self.linked_slider.posx = new_val end
		self.value = self.linked_slider.posx
		
		local domain = self.domain
		
		local new_color = get_slider_values(domain)
		if not new_color then return end
	
		local rgb = color2rgb(new_color, domain)
		if not rgb then return end
	
		active_color = rgb
		color_update()
	end

	local function edit_action(self, key, after)
		local new_val = tonumber(self.value)
		
		--validate text entry
		if key==string.byte'\r' then --return key
			edit_update(self)
			
			return iup.IGNORE
		else --check if edit value valid and if not copy value from slider
			if self.value ~= "" then
				local slider = self.linked_slider
				if not new_val or new_val>tonumber(slider.xmax) or new_val<tonumber(slider.xmin) then return iup.IGNORE end
			end
		end
	end
	
	local function palette_action(self, button, pressed, x, y, status)
		if button==iup.LMBUTTON and pressed==1 then
			x = x - tonumber(self.x)
			y = y - tonumber(self.y)
	
			local index = math.ceil(x/swatch_size) + 19*math.floor(y/swatch_size)
			local sel_color = hsv_palette[index]
			local r,g,b = string.match(sel_color, "^(%d+) (%d+) (%d+)$")
			
			if not r or not g or not b then return end
			
			active_color = {r,g,b}
			color_update()
		end
	end
	
	--common settings
	local ctrl_templates = {
		slider = {scrollbar="HORIZONTAL", size="300x", border="NO", expand="NO", dx=Font.Default, scroll_cb=slider_action},
		edit = {size=3*Font.Default, value=255, alignment="ARIGHT", nc=3, action=edit_action},
		label = {size=Font.Default, alignment="ARIGHT"},
	}
	
	 -------------------------
	--  Control Constructor  --
	 -------------------------

	for domain,components in pairs(color_domains) do
		local domain_box = {}
		local constructor
		for _,name in ipairs(components) do
			for ctrl_type,dest in pairs(cc) do
				local ctrl_settings = {} --create copy of table
				for k,v in pairs(ctrl_templates[ctrl_type] or {}) do
					ctrl_settings[k] = v
				end
			
				ctrl_settings.domain = domain
				ctrl_settings.name = name
			
				local constructor
				if ctrl_type=="slider" then
					constructor = iup.canvas
				
					ctrl_settings.xmin = slider_ranges[name][1]
					ctrl_settings.xmax = slider_ranges[name][2]
				elseif ctrl_type=="edit" then
					constructor = iup.text
				elseif ctrl_type=="label" then
					constructor = iup.label
				
					ctrl_settings.title = name:sub(1,1):upper()
				end
			
				if constructor then dest[name] = constructor(ctrl_settings) end --create control
			end
			
			--cross-link slider and edit text box
			cc.slider[name].linked_edit = cc.edit[name]
			cc.edit[name].linked_slider = cc.slider[name]
			
			local component_row = iup.hbox{
				cc.label[name],
				iup.fill{size=5},
				cc.slider[name],
				cc.edit[name],
				gap=6,
			}
			
			table.insert(domain_box, component_row)
		end
	
		domain_box.gap=6
		cc[domain] = iup.vbox(domain_box)
	end

	okbutton = iup.stationbutton{title ="OK"}
	resetbutton = iup.stationbutton{title="Reset", active="NO"}
	cancelbutton = iup.stationbutton{title ="Cancel"}

	color_preview = iup.text{title ="", size="64x64", expand="NO", border="NO", active="NO", bgcolor="0 255 0 255 &"}
	color_palette = iup.zbox{
		iup.label{title="", size=19*swatch_size.."x"..3*swatch_size, image="plugins/RadarColor/img/hsv_palette.png"},
		iup.canvas{button_cb=palette_action},
		expand="NO",
		all="YES",
	}

	local frame = iup.stationhighopacityframe{
		iup.stationhighopacityframebg{
			iup.vbox{
				iup.hbox{iup.fill{}, iup.label{title="Color Picker"}, iup.fill{}},
				iup.hbox{
					iup.vbox{
						iup.hbox{cc.RGB},
						iup.hbox{cc.HSV},
						gap=30,
					},
					iup.fill{},
					iup.vbox{iup.hbox{color_preview}},
					gap=20,
				},
				iup.hbox{iup.fill{size=8}, color_palette, iup.fill{}},
				iup.fill{size=6},
				iup.hbox{okbutton, cancelbutton, iup.fill{}, resetbutton, gap=15},
				gap=10,
				margin="5x5",
			},
			expand="NO",
		},
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				frame,
				iup.fill{},
			},
			iup.fill{},
		},
	
		BGCOLOR="0 0 0 160 *",
		FULLSCREEN="YES",
		RESIZE="NO",
		BORDER="NO",
		MENUBOX="NO",
		TOPMOST="YES",
		DEFAULTESC=cancelbutton,
		--DEFAULTENTER=okbutton,
	}

	function okbutton:action()
		color_update()
		if type(color_picker.color_cb)=="function" then
			color_picker.color_cb(active_color)
		end
		
		HideDialog(dlg)
	end

	function resetbutton:action()
		active_color = color_original
		color_update()
		
		self.active = "NO"
	end

	function cancelbutton:action()
		HideDialog(dlg)
	end

	function dlg:show_cb(data)
		active_color = color_original
		color_update()
		resetbutton.active = "NO"
	end

	dlg:map()
	
	color_picker.dlg = dlg
	return dlg
end

function color_picker:SetOriginalColor(rgb)
	assert(type(rgb)=="table") --TODO error msg
	
	for i=1,3 do
		color_original[i] = math.floor(math.min(math.max(tonumber(rgb[i]) or 0, 0), 255))
	end
	
	return unpack(color_original)
end

return color_picker