RadarColor = {}
RadarColor.version = '0.3'

--## Choco ##--

local color_list = dofile'color_list.lua'
local cpicker = dofile'color_picker.lua'
cpicker:create()

local color_list_sorted = {}
local color_list_max_length = 1 --calculate actual value later
for color_name,color_str in pairs(color_list) do
	table.insert(color_list_sorted, color_name)
	
	--find length of longest color name
	local length = string.len(color_name)
	if length > color_list_max_length then color_list_max_length = length end
end
table.sort(color_list_sorted)

--constants
local VARS_ID = "RadarSwap" --share tag with RadarSwap plug-in
local BLIP_COLORS = "blip_colors"
local BLIP_SHAPES = "blip_shapes"
local BLIP_SIZES = "blip_sizes"
local COLOR_LIST_COLUMNS = 4
local IMAGE_PATH = "plugins/RadarColor/img/"

local radar_items = {
	[0] = "target selection circle",
	[1] = nil, --unknown
	[2] = "own projectiles and mines",
	[3] = "homing projectiles following player",
	[4] = "friendly target",
	[5] = "hostile target",
	[6] = "other player projectiles and mines",
	[7] = "duelist",
	[8] = "group member",
	[9] = "roids and station objects",
	[10] = "powerup (cargo)",
	[11] = "station docking bay",
	[12] = "storm exit point",
	[13] = "regular exit point",
	[21] = "wormhole",
	[22] = "mostly hostile target",
	[23] = "mostly friendly target",
}
local MAX_BLIP_ID = 23
--! Using blip types 14-20 currently unstable (VO 1.8.384); use at own risk!
--! Adding non-valid entires to this list may cause VO client to crash

--! Do not change the order of these items, linked to save values
--! Use lower case only
local radar_shapes = {
	"round",
	"square",
	"diamond",
	"triangle",
	"triangle2",
	"plus",
	"x",
}
local radar_shapes_lookup = {}
for i,shape in ipairs(radar_shapes) do
	radar_shapes_lookup[shape] = i
end

local radar_items_sorted = {}
for k,_ in pairs(radar_items) do
	table.insert(radar_items_sorted, k)
end
table.sort(radar_items_sorted)

local help_str = '\127ffffffRadarColor v'..RadarColor.version..'  To set mine/projectile radar blip color use: /RadarColor \1270070c0R G B\127o (0-255 each) or /RadarColor default\127o'
local radar_left, radar_right --iup.radar

--Prints given string to chat box, appending plugin name to beginning and colors text white
local function doprint(str)
	if type(str)=="string" and str~="" then
		print("\127ffffffRadarColor: "..str.."\127o")
	elseif type(str)=="table" then
		local first_line = str[1]
		if type(first_line)~="string" or first_line=="" then return end
		
		first_line = "\127ffffffRadarColor: "..str[1]
		str[1] = first_line
		
		local full_text = table.concat(str, '\n')
		print(full_text..'\127o')
	end
end

--converts decimal color string "255 255 255" to hex "FFFFFF"
local function dec2hex_color(str)
	if type(str) ~= "string" then return end
	
	local r,g,b = string.match(str, "^(%x%x)(%x%x)(%x%x)$")
	r = r and tonumber(r, 16)
	g = g and tonumber(g, 16)
	b = b and tonumber(b, 16)
	if r and g and b then return str end --already correct format
	
	r,g,b = string.match(str, "^(%d+) (%d+) (%d+)$")
	r = tonumber(r)
	g = tonumber(g)
	b = tonumber(b)
	
	if r and g and b then
		r = math.min(r, 255)
		g = math.min(g, 255)
		b = math.min(b, 255)
		
		return string.format("%02X%02X%02X", r,g,b)
	else return end
end

--converts hex color string "FFFFFF" to decimal "255 255 255"
local function hex2dec_color(str)
	if type(str) ~= "string" then return end
	
	local r,g,b = string.match(str, "^(%d+) (%d+) (%d+)$") 
	r = tonumber(r)
	g = tonumber(g)
	b = tonumber(b)
	
	if r and g and b then
		r = math.min(r, 255)
		g = math.min(g, 255)
		b = math.min(b, 255)
		
		return string.format("%d %d %d", r,g,b)
	end
	
	r,g,b = string.match(str, "^(%x%x)(%x%x)(%x%x)$")
	r = r and tonumber(r, 16)
	g = g and tonumber(g, 16)
	b = b and tonumber(b, 16)
	
	if r and g and b then
		return string.format("%d %d %d", r,g,b)
	else return end
end

--Returns table of blip colors saved to config, e.g. {blip_id1 = color_str, ...}
--color strings are format "255 255 255"
--if id specified then only returns color string for that blip type (nil if no saved data)
local function read_blip_colors(id)
	local blip_colors = unspickle(gkini.ReadString(VARS_ID, BLIP_COLORS, "{}")) --load saved color from config file
	
	--only read colors for valid blip types
	local colors = {}
	for blip_id,_ in pairs(radar_items) do
		local hex_color = blip_colors[blip_id+1] --add 1 because id 0 is first entry
		local color_str = hex2dec_color(hex_color) --validate and convert color string
		
		if color_str then --valid, non-blank color
			colors[blip_id] = color_str
		end
	end
	
	id = tonumber(id)
	id = radar_items[id] and id --validate id
	if id then
		return colors[id] --returns single color string or nil
	else return colors end --returns table of all color strings
end

--Returns table of blip shapes saved to config, e.g. {blip_id1 = image_str, ...}
--image strings do not include file extension (.png appended automatically)
--if id specified then only returns image string for that blip type (nil if no saved data)
local function read_blip_shapes(id)
	local blip_shapes = unspickle(gkini.ReadString(VARS_ID, BLIP_SHAPES, "{}")) --load saved shapes from config file
	
	--only read colors for valid blip types
	local images = {}
	for blip_id,_ in pairs(radar_items) do
		local image_index = blip_shapes[blip_id+1] --add 1 because id 0 is first entry
		local image_str = radar_shapes[image_index]
		
		if image_str then --valid image name, ignore value of 0 for defaults
			images[blip_id] = image_str
		end
	end
	
	id = tonumber(id)
	id = radar_items[id] and id --validate id
	if id then
		return images[id] --returns single color string or nil
	else return images end --returns table of all color strings
end

--Returns table of blip sizes saved to config, e.g. {blip_id1 = size (number), ...}
--The default size is used when the size is set to 0
--if id specified then only returns size for that blip type (nil if no saved data)
local function read_blip_sizes(id)
	local blip_sizes = unspickle(gkini.ReadString(VARS_ID, BLIP_SIZES, "{}")) --load saved shapes from config file
	
	--only read colors for valid blip types
	local sizes = {}
	for blip_id,_ in pairs(radar_items) do
		local size = blip_sizes[blip_id+1] --add 1 because id 0 is first entry
		size = tonumber(size) or 0
		size = math.floor(math.max(size,0))
		
		if size>0 then sizes[blip_id] = size end
	end
	
	id = tonumber(id)
	id = radar_items[id] and id --validate id
	if id then
		return sizes[id] --returns single color string or nil
	else return sizes end --returns table of all color strings
end

--Update all iup.radar objects in radars table to blip_id/color_str pairs in colors table
--color strings are format "255 255 255" or "FFFFFF"; set color_str to blank string to use default color
--if no arguments specified, resets all blip types to default colors (interface reload required)
--! no validation done on color strings (must be done in external function)
local function set_blip_colors(radars, colors)
	assert(not colors or type(colors)=="table") --TODO error msg
	
	--create list of all blip types and set all to blank string by default (ensure every valid blip id has entry)
	local save_list = {} --color strings are format "FFFFFF"
	for i=0,MAX_BLIP_ID do --entry for each blip type
		table.insert(save_list, "")
	end
	
	--read saved blip colors
	if colors then --only load config settings if new colors specified, otherwise reset everything to defaults
		local saved_colors = read_blip_colors() --load existing save data (may not contain data for all blip types); color strings are format "255 255 255"
		for blip_id,color_str in pairs(saved_colors) do --any valid saved_colors entries replace save_list entries
			if radar_items[blip_id] then --ensure valid blip id
				save_list[blip_id+1] = dec2hex_color(color_str) --add 1 because id 0 is first entry
			end --ignore invalid blip ids
		end
	end
	
	--replace saved_colors entries with any valid entries from colors
	for blip_id,color_str in pairs(colors or {}) do
		if radar_items[blip_id] then --ensure valid blip id
			local hex_color = dec2hex_color(color_str) --convert color string to 6-digit hex string to reduce number of characters written to config
			save_list[blip_id+1] = hex_color or "" --add 1 because id 0 is first entry
			
			--update each radar's color for this blip type
			if hex_color then --non-blank color string 
				--if radars not defined then saves color value to config and will update on next HUD_SHOW event
				for _,radar in ipairs(radars or {}) do
					radar['BLIPCOLOR'..blip_id] = color_str --assigns new blip color to radars
				end
			end
		end
	end
	
	gkini.WriteString(VARS_ID, BLIP_COLORS, spickle(save_list)) --save one line to config for all blip colors
	
	return true
end

--Update all iup.radar objects in radars table to blip_id/image pairs in images table
--images strings are the name of the file without extension (.png gets appended)
--if no arguments specified, resets all blip types to default shapes (interface reload required)
--! no validation done on image strings (must be done in external function)
local function set_blip_shapes(radars, images)
	assert(not images or type(images)=="table") --TODO error msg
	
	--create list of all blip types and set all to zero by default (ensure every valid blip id has entry)
	local save_list = {}
	for i=0,MAX_BLIP_ID do --entry for each blip type
		table.insert(save_list, 0)
	end
	
	--read saved blip images
	if images then --only load config settings if new shapes specified, otherwise reset everything to defaults
		local saved_shapes = read_blip_shapes() --load existing save data (may not contain data for all blip types)
		for blip_id,image_str in pairs(saved_shapes) do --any valid saved_shapes entries replace save_list entries
			if radar_items[blip_id] then --ensure valid blip id
				save_list[blip_id+1] = radar_shapes_lookup[image_str] or 0 --convert to index to conserve space
			end --ignore invalid blip ids
		end
	end
	
	--replace saved_shapes entries with any valid entries from images
	for blip_id,image_str in pairs(images or {}) do
		if radar_items[blip_id] then --ensure valid blip id
			local image_index = radar_shapes_lookup[image_str]
			save_list[blip_id+1] = radar_shapes_lookup[image_str] or 0 --add 1 because id 0 is first entry; 0 for default
			
			--update each radar's image for this blip type
			if image_index then --non-default value 
				--if radars not defined then saves image name to config and will update on next HUD_SHOW event
				for _,radar in ipairs(radars or {}) do
					radar['BLIPTEXTURE'..blip_id] = IMAGE_PATH..image_str..'.png' --assigns new blip image to radars
				end
			end
		end
	end
	
	gkini.WriteString(VARS_ID, BLIP_SHAPES, spickle(save_list)) --save one line to config for all blip shapes
	
	return true
end

--Update all iup.radar objects in radars table to blip_id/size pairs in sizes table
--The size is the number of pixels for the radar blip
--if no arguments specified, resets all blip types to default sizes (interface reload required)
--! no validation done on size values (must be done in external function)
local function set_blip_sizes(radars, sizes)
	assert(not sizes or type(sizes)=="table") --TODO error msg
	
	--create list of all blip types and set all to zero by default (ensure every valid blip id has entry)
	local save_list = {}
	for i=0,MAX_BLIP_ID do --entry for each blip type
		table.insert(save_list, 0)
	end
	
	--read saved blip images
	if sizes then --only load config settings if new sizes specified, otherwise reset everything to defaults
		local saved_sizes = read_blip_sizes() --load existing save data (may not contain data for all blip types)
		for blip_id,size in pairs(saved_sizes) do --any valid saved_sizes entries replace save_list entries
			if radar_items[blip_id] then --ensure valid blip id
				save_list[blip_id+1] = size
			end --ignore invalid blip ids
		end
	end
	
	--replace saved_sizes entries with any valid entries from sizes
	for blip_id,size in pairs(sizes or {}) do
		if radar_items[blip_id] then --ensure valid blip id
			size = tonumber(size) or 0 --value of zero uses default
			save_list[blip_id+1] = math.floor(math.max(size,0)) --add 1 because id 0 is first entry; --non-integer values valid but take more space in config file saved data
			
			--update each radar's size for this blip type
			if size>0 then --non-default value
				--if radars not defined then saves image name to config and will update on next HUD_SHOW event
				for _,radar in ipairs(radars or {}) do
					radar['BLIPSIZE'..blip_id] = size --assigns new blip image to radars
				end
			end
		end
	end
	
	gkini.WriteString(VARS_ID, BLIP_SIZES, spickle(save_list)) --save one line to config for all blip shapes
	
	return true
end

--Reads saved blip colors from config and assigns to radars
function RadarColor:HUD_SHOW(event, data)
	--reference latest radar instances
	radar_left = HUD.radarlayer[2][1][2]
	radar_right = HUD.radarlayer[2][3][2]
	
	local colors = read_blip_colors() --color strings are format "255 255 255"
	RadarColor.cc = colors
	if next(colors) then --at least one color specified
		set_blip_colors({radar_left, radar_right}, colors)
	end
	
	local shapes = read_blip_shapes()
	RadarColor.img = shapes
	if next(shapes) then --at least one color specified
		set_blip_shapes({radar_left, radar_right}, shapes)
	end
	
	local sizes = read_blip_sizes()
	RadarColor.ss = sizes
	if next(sizes) then --at least one color specified
		set_blip_sizes({radar_left, radar_right}, sizes)
	end
end

--Sets radar color of specified blip type (number 0-23) to specified RGB color string
--Instead of specifiying a color string: "default" resets to default color, "palette" uses last selected color from ship puchase color palette
--Color string may be specified as 3 separate color values 0-255 or as a single string separating color values with spaces, e.g. "255 0 128"
function RadarColor:blipcolor(args)
	--convert all args to lower case
	args = args or {}
	for i,arg in ipairs(args) do
		args[i] = tostring(arg):lower()
	end
	
	--first argument is blip_id
	local blip_id = args[1]
	if blip_id == "all" then
		if args[2] == "default" then
			set_blip_colors() --reset colors to default
			doprint"All blip types reset to default colors (requires interfaces reload)"
			
			return true
		else
			RadarColor:help"blipcolor" --"all" can only be used for setting defaults
			return false
		end
	else blip_id = tonumber(blip_id) end
	
	--handle cases where blip_id is not valid
	if not blip_id then
		RadarColor:help"blipcolor"
		return false
	elseif not radar_items[blip_id] then
		doprint(string.format("Blip type %G is not valid", blip_id))
		return false
	end
	
	--second argument can be a keyword or color string
	local rgb = color_list[ args[2] ] --color string if valid color name specified, else nil
	if args[2]=="default" then
		local is_success = set_blip_colors({}, {[blip_id]=""}) --removes saved data for blip_id
		if is_success then doprint("Blip type "..tostring(blip_id).." changed to default color (interface reload required)"); return true end
		return
	--elseif args[2]=="palette" then
	--	rgb = ShipPalette_string[(GetShipPurchaseColor() or 0)+1] --last selected color from ship purchase palette
	elseif args[2]=="picker" then
		local prev_color = read_blip_colors(blip_id) or "0 0 0"
		
		cpicker:SetOriginalColor{string.match(prev_color, "^(%d+) (%d+) (%d+)$")}
		cpicker.color_cb = function(color_table)
			rgb = string.format("%d %d %d", unpack(color_table))
			
			local is_success = set_blip_colors({radar_left,radar_right}, {[blip_id]=rgb})
			if is_success then doprint("Blip type "..tostring(blip_id).." changed to color "..rgbtohex(rgb)..rgb..'\127o') end
		end
		
		ShowDialog(cpicker.dlg, iup.CENTER, iup.CENTER)
		
		return
	elseif not rgb then rgb = hex2dec_color(args[2]) end --assume first argument is color string, nil if not the case
	
	if not rgb then --assume first 3 args are r,g,b values
		local r,g,b = tonumber(args[2]), tonumber(args[3]), tonumber(args[4])
		
		if r and g and b then
			rgb = hex2dec_color(string.format("%d %d %d", r, g, b)) --guaranteed to result in valid color string
		else
			RadarColor:help"blipcolor" --invalid input
			return false
		end
	end
	
	local is_success = set_blip_colors({radar_left,radar_right}, {[blip_id]=rgb})
	if is_success then doprint("Blip type "..tostring(blip_id).." changed to color "..rgbtohex(rgb)..rgb..'\127o'); return true end
end

function RadarColor:blipshape(args)
	--convert all args to lower case
	args = args or {}
	for i,arg in ipairs(args) do
		args[i] = tostring(arg):lower()
	end
	
	--first argument is blip_id
	local blip_id = args[1]
	if blip_id == "all" then
		if args[2] == "default" then
			set_blip_shapes() --reset colors to default
			doprint"All blip types reset to default shapes (requires interfaces reload)"
			
			return true
		else
			RadarColor:help"blipshape" --"all" can only be used for setting defaults
			return false
		end
	else blip_id = tonumber(blip_id) end
	
	--handle cases where blip_id is not valid
	if not blip_id then
		RadarColor:help"blipshape"
		return false
	elseif not radar_items[blip_id] then
		doprint(string.format("Blip type %G is not valid", blip_id))
		return false
	end
	
	--second argument is the name of the image to use
	local image = args[2] or ""
	if image=="default" then
		local is_success = set_blip_shapes({}, {[blip_id]=""}) --removes saved shapes data for blip id
		if is_success then doprint("Blip type "..tostring(blip_id).." changed to default shape (interface reload required)"); return true end
		return false
	elseif not radar_shapes_lookup[image] then --invalid image name
		doprint(string.format("Blip shape %s is not valid", image))
		return false
	end
	
	local is_success = set_blip_shapes({radar_left,radar_right}, {[blip_id]=image})
	if is_success then doprint("Blip type "..tostring(blip_id).." changed to shape "..image..'\127o'); return true end
end

function RadarColor:blipsize(args)
	--convert all args to lower case
	args = args or {}
	for i,arg in ipairs(args) do
		args[i] = tostring(arg):lower()
	end
	
	--first argument is blip_id
	local blip_id = args[1]
	if blip_id == "all" then
		if args[2] == "default" then
			set_blip_sizes() --reset colors to default
			doprint"All blip types reset to default sizes (requires interfaces reload)"
			
			return true
		else
			RadarColor:help"blipsize" --"all" can only be used for setting defaults
			return false
		end
	else blip_id = tonumber(blip_id) end
	
	--handle cases where blip_id is not valid
	if not blip_id then
		RadarColor:help"blipsize"
		return false
	elseif not radar_items[blip_id] then
		doprint(string.format("Blip type %G is not valid", blip_id))
		return false
	end
	
	--second argument is the size in pixels to make the blip type
	local size = args[2]
	if size=="default" then
		local is_success = set_blip_sizes({}, {[blip_id]=0}) --removes saved size for blip id
		if is_success then doprint("Blip type "..tostring(blip_id).." changed to default size (interface reload required)"); return true end
		return false
	end
	
	--validate size is a number
	local size_val = tonumber(size)
	if not size then
		doprint(string.format("Blip size %s is not valid", size))
		return false
	end
	
	size_val = math.floor(math.max(size_val,0)) --Coerce size to positive integer
	
	local is_success = set_blip_sizes({radar_left,radar_right}, {[blip_id]=size})
	if is_success then doprint("Blip type "..tostring(blip_id).." changed to size "..size..'\127o'); return true end
end

--Resets all blip types to default color (requires interface reload)
function RadarColor:defaults(args)
	set_blip_colors()
	set_blip_shapes()
	set_blip_sizes()
	doprint"All blip types reset to default colors, shapes and sizes (requires interfaces reload)"
	return true
end

local list_options = {plain=true}
function RadarColor:list(args)
	--find all options specified
	local options = {}
	for i,arg in ipairs(args or {}) do
		local arg_str = tostring(arg):lower()
		
		if list_options[arg_str] and not options[arg_str] then --specified option is valid (duplicates ignored)
			options[arg_str] = i
		end
	end
	
	local list_items = radar_items_sorted --blip types to be listed, assume full list by default
	
	--first arg can be a number representing blip type
	local blip_id = tonumber(args[1])
	if radar_items[blip_id] then
		list_items = {blip_id} --only list 1 blip type
	end
	
	local keyword = args[1] and tostring(args[1]):lower() --first arg can name type of list, list blip types by default
	local use_color = not options['plain']
	
	local color_end = use_color and '\127o' or ''
	local default_color = use_color and '\127808080' or ''
	
	--generate text to be listed
	local list_lines --text to be printed
	if keyword=="colors" then --list all possible color names from color_list
		local num_colors = #color_list_sorted
		local num_columns = math.floor(math.max(tonumber(COLOR_LIST_COLUMNS) or 1, 1)) --protect against invalid value for COLOR_LIST_COLUMNS
		local num_rows = math.ceil(num_colors/num_columns)
		
		list_lines = {'Available color names:'} --first line
		for row=1,num_rows do
			local current_line = {}
			for col=1,num_columns do
				local i = row + (col-1)*num_rows
				local word = color_list_sorted[i] or ""
				local word_length = string.len(word)
				
				local color_code = rgbtohex(color_list[word])
				local whitespace = string.rep(" ", color_list_max_length - word_length)
				
				table.insert(current_line, (use_color and color_code or '')..word..whitespace)
			end
			
			table.insert(list_lines, '\t'..table.concat(current_line, '    '))
		end
	elseif keyword=="shapes" then --list all available shape names
		list_lines = {'Available shape names:'} --first line
		for _,shape in ipairs(radar_shapes) do
			table.insert(list_lines, '\t'..shape)
		end
		table.insert(list_lines, '\tdefault') --append "default" to bottom of list
	else
		--invalid first option specified, show help instead; invalid options beyond the first are ignored
		if keyword and not list_options[keyword] and not blip_id then
			RadarColor:help"list"
			return false
		end
	
		list_lines = {'Radar blip color assignments: (color, shape, size)'} --first line
		for _,blip_type in ipairs(list_items or {}) do
			local desc = radar_items[blip_type]
			local color_str = read_blip_colors()[blip_type]
			local color_code = use_color and color_str and rgbtohex(color_str) or ''
			color_str = color_str or default_color.."DEFAULT"..color_end --if set to default, actual color is not known
			
			local shape_str = read_blip_shapes()[blip_type] or default_color.."DEFAULT"..color_end
			local blip_size = read_blip_sizes()[blip_type] or default_color.."DEFAULT"..color_end
			
			table.insert(list_lines, string.format("\t%u: %s (%s%s%s, %s, %s)", blip_type, tostring(desc), color_code, color_str, string.len(color_code)>0 and color_end or '', shape_str, blip_size)) --line for each blip type displayed
		end
	end
	
	doprint(list_lines)
	return true
end

--function RadarColor:picker(args)
--	ShowDialog(cpicker.dlg, iup.CENTER, iup.CENTER)
--end

local help_text= {
	['help']='\127ffffffRadarColor v'..RadarColor.version..'  Available commands are: (for more info use /RadarColor help \127ffff00COMMAND\127o)'..
		'\n\t/RadarColor \127808080[\127oblipcolor\127808080]\127o \1270070c0ID\127o \127ffff00Color\127o - Changes color of radar blips of type \1270070c0ID\127o to \127ffff00Color\127o'..
		'\n\t/RadarColor blipshape \1270070c0ID\127o \127ffff00Shape\127o - Changes shape of radar blips of type \1270070c0ID\127o to \127ffff00Shape\127o'..
		'\n\t/RadarColor blipsize \1270070c0ID\127o \1270070c0N\127o - Changes size of radar blips of type \1270070c0ID\127o to \1270070c0N\127o pixels'..
		'\n\t/RadarColor list \127808080[\127otype\127808080] [\127ooptions\127808080]\127o - Prints list of items. Valid list types are "colors" or blip types by default.'..
		'\n\t/RadarColor defaults - Restores all blip types to default colors (takes effect on next interface reload)',
	['blipcolor']='\127ffffff/RadarColor \127808080[\127oblipcolor\127808080]\127o \1270070c0ID\127o \127ffff00Color\127o - Changes color of radar blips of type \1270070c0ID\127o to \127ffff00Color\127o'..
		'\n\tNote that the term blipcolor may be omitted. For a list of possible blip types, use: /RadarColor list'..
		'\n\t\127ffff00Color\127o can be an RGB color of the format "255 255 255" or "FFFFFF" or even specified by name (e.g. "red")'..
		'\n\tFor full list of color names, use: /RadarColor list colors'..
		'\n\tTo bring up a color picker for easier color selection, use /RadarColor \1270070c0ID\127o picker'..
		'\n\tReset colors of all blip types to defaults with /RadarColor \127808080[\127oblipcolor\127808080]\127o all defaults',
	['blipshape']='\127ffffff/RadarColor blipshape \1270070c0ID\127o \127ffff00Shape\127o - Changes shape of radar blips of type \1270070c0ID\127o to \127ffff00Shape\127o'..
		'\n\tFor a list of possible blip types, use: /RadarColor list'..
		'\n\tFor list of available blip shapes, use: /RadarColor list shapes',
		'\n\tReset shapes of all blip types to defaults with /RadarColor blipshape all defaults',
	['blipsize']='\127ffffff/RadarColor blipsize \1270070c0ID\127o \1270070c0N\127o - Changes size of radar blips of type \1270070c0ID\127o to \1270070c0N\127o pixels'..
		'\n\tReset sizes of all blip types to defaults with /RadarColor blipsize all defaults',
	['list']='\127ffffff/RadarColor list \127808080[\127otype\127808080] [\127ooptions\127808080]\127o - Prints list of items. Valid list types are "colors", "shapes" or blip types by default.'..
		'\n\tThe list type can also be a blip type ID to only show that one along with its color/shape/size settings (e.g. /RadarColor list \1270070c0ID\127o)'..
		'\n\tThe "plain" option prints out the list in white text only (e.g. /RadarColor list colors plain)',
	['defaults']='\127ffffff/RadarColor defaults - Restores all blip types to default colors, shapes and sizes (takes effect on next interface reload)'..
		'\n\tTo reload the interface without restarting the client, type into the console: /lua ReloadInterface()'..
		'\n\tTo open the console, use the command /consoletoggle or press the ` key',
	['command']='\127ffffffTo get more information about how to use a command, type: /RadarColor help \127ffff00COMMAND\127o'..
		'\n\tWhere \127ffff00COMMAND\127o is the command you want to get help on (e.g. /RadarColor help list)',
}

--console help text
function RadarColor:help(args)
	args = args or {}
	
	print(args[1] and help_text[args[1]:lower()] or help_text.help)
end

local inputs = {list=true, blipcolor=true, blipsize=true, blipshape=true, defaults=true, help=true}
function RadarColor:input(args)
	args = args or {}
	local command = table.remove(args, 1)
	local cmd_num = tonumber(command)
	command = command and tostring(command):lower()
	
	if inputs[command] then --process valid command
		RadarColor[command](RadarColor, args)
	elseif cmd_num then --for convenience, blipcolor command can be omitted
		table.insert(args, 1, command) --replace command as first arg
		RadarColor:blipcolor(args)
	else RadarColor:help() end --invalid command, show help
end


RegisterUserCommand("RadarColor", RadarColor.input)
RegisterEvent(RadarColor, "HUD_SHOW")